@@ -35,6 +35,7 @@
 #include <QToolButton>
 #include <QPointer>
 #include <QGridLayout>
+#include <QFile>
 
 // KDE Related Includes
 #include <KActionCollection>
@@ -154,13 +155,12 @@ MainWindow::~MainWindow()
 
 void MainWindow::closeEvent(QCloseEvent *event)
 {
-    switch (saveIfChanged()) {
-    case KMessageBox::Cancel :
-        event->ignore();
-        return;
-    default:
+    if (queryClose() == true) {
         event->accept();
         return;
+    } else {
+        event->ignore();
+        return;
     }
 }
 
@@ -355,7 +355,7 @@ QWidget* MainWindow::setupSidePanel()
 void MainWindow::setupActions()
 {
     kDebug() << "create and connect actions";
-    KStandardAction::quit(kapp, SLOT(quit()), actionCollection());
+    KStandardAction::quit(this, SLOT(quit()), actionCollection());
     KStandardAction::preferences(this, SLOT(showSettings()), actionCollection());
 
     // setup graph visual editor actions and add them to mainwindow action collection
@@ -386,8 +386,6 @@ void MainWindow::setupActions()
     createAction("document-export", i18nc("@action:inmenu", "Export Script as"),      "export-script-as",      SLOT(saveActiveScriptAs()), _codeEditor);
     createAction("",  i18nc("@action:inmenu", "Loaded Plugins"),      "loaded-plugins",      SLOT(showLoadedPlugins()), this);
     createAction("",  i18nc("@action:inmenu", "Configure Code Editor..."),      "config-code-editor",      SLOT(showCodeEditorConfig()), this);
-
-    KStandardAction::quit(kapp, SLOT(quit()),  actionCollection());
 }
 
 void MainWindow::createAction(const QByteArray& iconName, const QString& actionTitle, const QString& actionName,
@@ -395,7 +393,7 @@ void MainWindow::createAction(const QByteArray& iconName, const QString& actionT
 {
     KAction* action = new KAction(KIcon(iconName), actionTitle, parent);
     action->setShortcut(shortcut);
-    action->setShortcutContext(Qt::WidgetShortcut);
+    action->setShortcutContext(Qt::ApplicationShortcut);
     actionCollection()->addAction(actionName, action);
     connect(action, SIGNAL(triggered(bool)), parent, slot);
 }
@@ -404,7 +402,6 @@ void MainWindow::createAction(const QByteArray& iconName, const QString& actionT
                               const char* slot, QObject *parent)
 {
     KAction* action = new KAction(KIcon(iconName), actionTitle, parent);
-    action->setShortcutContext(Qt::WidgetShortcut);
     actionCollection()->addAction(actionName, action);
     connect(action, SIGNAL(triggered(bool)), parent, slot);
 }
@@ -585,7 +582,7 @@ void MainWindow::loadDocument(const QString& name)
 
 void MainWindow::createNewProject()
 {
-    if (saveIfChanged() == KMessageBox::Cancel) {
+    if (!queryClose()) {
         return;
     }
 
@@ -597,6 +594,8 @@ void MainWindow::createNewProject()
     _currentProject->addCodeFileNew(_codeEditor->newScript());
     _currentProject->addGraphFileNew(DocumentManager::self().newDocument());
     _journalWidget->openJournal(_currentProject);
+    _currentProject->setModified(false);
+
     updateCaption();
 }
 
@@ -657,7 +656,7 @@ void MainWindow::saveProjectAs()
 
 void MainWindow::openProject(const KUrl& fileName)
 {
-    if (saveIfChanged() == KMessageBox::Cancel) {
+    if (!queryClose()) {
         return;
     }
 
@@ -729,28 +728,58 @@ void MainWindow::updateCaption()
     setCaption(caption);
 }
 
+QString MainWindow::uniqueFilename(const QString &basePrefix, const QString &suffix) {
+    QFile targetFile;
+    QString basePath = _currentProject->projectDirectory();
+    QString fullSuffix = "." + suffix;
+    QString fullPrefix = basePrefix;
+
+    if (fullPrefix.isNull()) {
+        fullPrefix = _currentProject->projectFile().fileName().remove(QRegExp(".rocsz*$"));
+    } else if (fullPrefix.endsWith(fullSuffix)) {
+        fullPrefix.remove(QRegExp(fullSuffix + "$"));
+    }
+
+    targetFile.setFileName(basePath + fullPrefix + fullSuffix);
+    for(int i = 1; targetFile.exists(); i++) {
+        targetFile.setFileName(basePath + fullPrefix + QString::number(i) + fullSuffix);
+    }
+
+    return targetFile.fileName();
+}
+
 void MainWindow::saveScripts()
 {
-    QString fileName = _currentProject->projectFile().fileName().remove(QRegExp(".rocsz*$"));
+    foreach (KTextEditor::Document * textDocument, _currentProject->codeFilesNew()) {
+        QString basePrefix = KInputDialog::getText(i18n("ScriptName"), i18n("Enter the name of your new script"));
+        QString fileName = uniqueFilename(basePrefix, "js");
 
-    foreach(KTextEditor::Document * textDocument, _currentProject->codeFilesNew()) {
-        QString file = _currentProject->projectDirectory() + fileName + ".js";
-        textDocument->saveAs(KUrl::fromLocalFile(file));
-        _currentProject->saveCodeFileNew(textDocument, file);
+        textDocument->saveAs(KUrl::fromLocalFile(fileName));
+        _currentProject->saveCodeFileNew(textDocument, fileName);
     }
+
     _codeEditor->saveAllScripts();
 }
 
 void MainWindow::newScript()
 {
-    QString file = KInputDialog::getText(i18n("ScriptName"), i18n("Enter the name of your new script"));
-    if (file.isEmpty()) {
-        kDebug() << "Filename is empty and no script file was created.";
-        return;
+    if (!_currentProject->isTemporary()) {
+        QString basePrefix = KInputDialog::getText(i18n("ScriptName"), i18n("Enter the name of your new script"));
+        if (basePrefix.isNull()) {
+            kDebug() << "Filename is empty and no script file was created.";
+        } else {
+            QString fileName = uniqueFilename(basePrefix, "js");
+
+            KTextEditor::Document *document = _codeEditor->newScript(KUrl::fromLocalFile(fileName));
+            _currentProject->addCodeFileNew(document);
+            _currentProject->saveCodeFileNew(document, KUrl::fromLocalFile(fileName));
+        }
+    } else {
+        saveProject(true);
+        if (!_currentProject->isTemporary()) {
+            newScript();
+        }
     }
-    if (!file.endsWith(QLatin1String(".js")))
-        file.append(".js");
-    _codeEditor->newScript(KUrl::fromLocalFile(file));
 }
 
 void MainWindow::saveGraph(Document* document)
@@ -812,22 +841,13 @@ void MainWindow::newGraph()
     _currentProject->addGraphFile(file);
 }
 
-int MainWindow::saveIfChanged()
+bool MainWindow::queryClose()
 {
     if (!_currentProject) {
-        return KMessageBox::No;
-    }
-
-    if (_currentProject->isModified() || _journalWidget->isModified()) {
-        const int btnCode = KMessageBox::warningYesNoCancel(this, i18nc(
-                                "@info",
-                                "Changes on your project are unsaved. Do you want to save your changes?"));
-        if (btnCode == KMessageBox::Yes) {
-            saveProject();
-        }
-        return btnCode;
+        return true;
     }
 
+    //TODO move modification information to project
     bool anyGraphDocumentModified = false;
     foreach(Document * document, DocumentManager::self().documentList()) {
         if (document->isModified()) {
@@ -835,34 +855,34 @@ int MainWindow::saveIfChanged()
             break;
         }
     }
-    if (anyGraphDocumentModified && !_codeEditor->isModified()) {
+
+    if (_currentProject->isModified()
+        || _journalWidget->isModified()
+        || anyGraphDocumentModified
+        || _codeEditor->isModified())
+    {
         const int btnCode = KMessageBox::warningYesNoCancel(this, i18nc(
                                 "@info",
-                                "Changes on your graph document are unsaved. Do you want to save your changes?"));
+                                "Changes on your project are unsaved. Do you want to save your changes?"));
         if (btnCode == KMessageBox::Yes) {
             saveProject();
+            return true;
         }
-        return btnCode;
-    }
-    if (!DocumentManager::self().activeDocument()->isModified() && _codeEditor->isModified()) {
-        const int btnCode = KMessageBox::warningYesNoCancel(this, i18nc(
-                                "@info",
-                                "Changes on your script files are unsaved. Do you want to save all unsaved scripts?"));
-        if (btnCode == KMessageBox::Yes) {
-            saveProject();
+        if (btnCode == KMessageBox::No) {
+            return true;
         }
-        return btnCode;
+        // do not close
+        return false;
     }
-    if (anyGraphDocumentModified && _codeEditor->isModified()) {
-        const int btnCode = KMessageBox::warningYesNoCancel(this, i18nc(
-                                "@info",
-                                "Changes on your script files and on your graph document are unsaved. Do you want to save your graph document and all unsaved scripts?"));
-        if (btnCode == KMessageBox::Yes) {
-            saveProject();
-        }
-        return btnCode;
+    // save to close project: no changes
+    return true;
+}
+
+void MainWindow::quit()
+{
+    if (queryClose()) {
+        kapp->quit();
     }
-    return KMessageBox::No;
 }
 
 void MainWindow::importGraphFile()
